/*
 * Copyright (c) 2015-2016 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 *                                      Anthony Mallet on Wed Jul  6 2016
 */
#ifndef H_DYNAMIXEL_CODELS
#define H_DYNAMIXEL_CODELS

#include <errno.h>
#include <math.h>
#include <stdbool.h>
#include <stdio.h>
#include <string.h>
#include <termios.h>

#include "dynamixel_c_types.h"

struct dynamixel_channel_s {
  char path[1024];
  int fd;

  uint8_t buf[128], r, w; /* read ring buffer */

  uint8_t state;
  uint8_t id, len, error, p, param[256], sum;
};

enum dx_cmd {
  DX_PING =		1,
  DX_READ =		2,
  DX_WRITE =		3,
  DX_SYNC_WRITE =	0x83,
  DX_BULK_READ =	0x92
};

enum dx_ref {
  DX_MODEL_NUMBER =		0x00,
  DX_RETURN_DELAY_TIME =	0x05,
  DX_CW_ANGLE_LIMIT =		0x06,
  DX_STATUS_RETURN_LEVEL =	0x10,
  DX_D_GAIN =			0x1a,
  DX_GOAL_POSITION =		0x1e,
  DX_MOVING_SPEED =		0x20,
  DX_TORQUE_LIMIT =		0x22,
  DX_PRESENT_POSITION =		0x24,
  DX_TORQUE_CONTROL_MODE =	0x46,
  DX_GOAL_TORQUE =		0x47
};

#define DX_POSITION_RESOLUTION	(0.088 * M_PI/180.)
#define DX_VELOCITY_RESOLUTION	(0.114 / 60.)
#define DX_TORQUE_RESOLUTION	(0.1)

static inline genom_event
dx_e_sys_error(const char *s, genom_context self)
{
  dynamixel_e_sys_detail d;
  size_t l = 0;

  d.code = errno;
  if (s) {
    strncpy(d.what, s, sizeof(d.what) - 3);
    l = strlen(s);
    strcpy(d.what + l, ": ");
    l += 2;
  }
  strerror_r(d.code, d.what + l, sizeof(d.what) - l);
  return dynamixel_e_sys(&d, self);
}

int	dx_open_tty(const char *device, speed_t baud);
int	dx_wait_msg(const struct dynamixel_channel_s *channel, int timeout);
int	dx_recv_msg(struct dynamixel_channel_s *chan, bool block);
int	dx_send_msg(const struct dynamixel_channel_s *chan, uint8_t id,
                enum dx_cmd cmd, ...);

#endif /* H_DYNAMIXEL_CODELS */
